
#' pred_rfci_eaamm2010_prime

#' pred_rfci_eaamm2010_prime
#' Adaptation of method from EAA2010 with Ranger
#' Method from Guide to the demonstration of equivalence of ambient air monitoring methods, expression (7.30) to (7.32)
#' https://ec.europa.eu/environment/air/quality/legislation/pdf/equivalence.pdf
#' @param model_rf Ranger RF model only
#' @param dataset dataset with target, Predictors
#' @param target column name of the target
#' @param phi_g1 first parameter value of the output error
#' @param phi_g2 first parameter value of the output error
#' @param phi_g3 first parameter value of the output error
#' @keywords cipred
#' @export
#' @examples
#' \dontrun{
#' pred_rfci_eaamm2010_prime()
#' }
#
pred_rfci_eaamm2010_prime <- function(model_rf,dataset,target,phi_g1=5.64/1.96,phi_g2=5/100/1.96,phi_g3=112.8){

  tmp <- predict_eaam2010_prime(model_rf=model_rf,dataset=dataset,target=target,phi_g1=phi_g1,phi_g2=phi_g2,phi_g3=phi_g3)

  res <- data.frame(obs=tmp$obs,
                    prediction=tmp$prediction,
                    se_g=tmp$se_g,
                    se_hr=tmp$se_hr,
                    se = tmp$se_tot,
                    lower_pred_0_05 = tmp$prediction - 1.96*tmp$se_tot,
                    upper_pred_0_05 = tmp$prediction + 1.96*tmp$se_tot,
                    pvalue=tmp$pvalue,
                    method = tmp$method)

  return(res)
}


#' predict_eaam2010_prime

#' predict_eaam2010_prime
#' Adaptation of method from EAA2010 with Ranger
#' Method from Guide to the demonstration of equivalence of ambient air monitoring methods, expression (7.30) to (7.32)
#' https://ec.europa.eu/environment/air/quality/legislation/pdf/equivalence.pdf
#' @param model_rf Ranger RF model only
#' @param dataset dataset with target, Predictors
#' @param target column name of the target
#' @param phi_g1 first parameter value of the output error
#' @param phi_g2 first parameter value of the output error
#' @param phi_g3 first parameter value of the output error
#' @keywords cipred
#' @export
#' @examples
#' \dontrun{
#' predict_eaam2010_prime()
#' }
#
predict_eaam2010_prime <- function(model_rf,dataset,target,phi_g1,phi_g2,phi_g3){

  tmp <- predict_ranger(model_rf,dataset)

  # intercept and slope using orthogonal regression
  beta <- sor(reference=dataset[[target]],client=tmp$predictions)

  #sum of the squared residuals of the orthogonal regression
  rss_v <- rss(reference=dataset[[target]],client=tmp$predictions)

  n <- length(dataset[[target]])

  se_g <- expr_sig_g_ts_3par(y_tilde_=dataset[[target]],phi_g1=phi_g1,phi_g2=phi_g2,phi_g3=phi_g3)
  se_hr <- sqrt( (rss_v / (n-2)) - se_g^2 + (beta[1] + (beta[2] -1) * dataset[[target]])^2  )
  se_tot <- se_g + se_hr

  res <- data.frame(obs=dataset[[target]],
                    prediction=tmp$predictions,
                    se_g=se_g,
                    se_hr=se_hr,
                    se_tot = se_tot,
                    pvalue = stats::pnorm(q=dataset[[target]], mean=tmp$predictions,sd=se_tot),
                    method          = rep("ranger_eaam2010_prime",length(tmp$predictions)))

  return(res)
}


#orthogonal regression
sor <- function(reference,client){
  # Use principal component analysis
  v <- stats::prcomp(cbind(client,reference))$rotation
  # Slope from the PCA output
  slope <- v[2,1] / v[1,1]
  # Intercept
  intercept <- mean(reference, na.rm=TRUE) - slope * mean(client, na.rm=TRUE)
  res <- cbind(intercept, slope)
  return(res)
}

#sum of the squared residuals of the orthogonal regression
rss <-function(reference,client){
  beta <- sor(reference,client)
  res <- sum( (client - beta[1] - beta[2] * reference)^2 , na.rm=TRUE)
  return(res)
}

# compute the standard deviation parameter of the output error
expr_sig_g_ts_3par <- function(y_tilde_,phi_g1,phi_g2,phi_g3){
    res <- rep(NA,length(y_tilde_))
    res[which(y_tilde_ <= abs(phi_g3))] <- abs(phi_g1)
    res[which(y_tilde_ > abs(phi_g3))] <- abs(phi_g2)*y_tilde_[which(y_tilde_ > abs(phi_g3))]
    return(res)
}
