
library(dplyr)

########################
# PARAMETERS           #
########################
element_id <- c("NO2")

# keeping the interesting stations
#station_id <- c("7","464","827","665","9","11","163","504","809")
station_id <- c("7","464","827","665","9","11","163","809")

timeResolution <- "hourly"
precision <- "hourly"

#############################
## Training/testing dataset #
#############################

# Period 2017-2018
fromDate <- "2017/01/01 00:00:00"
toDate <- "2018/12/31 23:00:00"

# luft
tmp <- data.luftkval.oslo10::get_short_data(stations= station_id,
                referencetimeFrom=fromDate,
                referencetimeTo=toDate,
                timeResolution=timeResolution,
                elements=element_id)

luft <- friendlyts::ts_spread(df=tmp, col_date=c("date"),
                               col_key=c("CO_NAME","ID"),col_target="value",
                               precision=timeResolution,tzone="UTC",
                               date_type="posixlt")

luft <- friendlyts::check_date(luft,
    fromDate=fromDate,
    toDate=toDate,
    timeResolution=timeResolution,
    v=1,
    precision=timeResolution,
    tzone="UTC",
    missingValues=NA,
    date_type="posixlt")

# luft qa
luft_qa <- friendlyts::ts_spread(df=tmp, col_date=c("date"),
                               col_key=c("CO_NAME","ID"),col_target="qualityCode",
                               precision=timeResolution,tzone="UTC",
                               date_type="posixlt")

luft_qa <- friendlyts::check_date(luft_qa,
    fromDate=fromDate,
    toDate=toDate,
    timeResolution=timeResolution,
    v=1,
    precision=timeResolution,
    tzone="UTC",
    missingValues=NA,
    date_type="posixlt")

icol <- friendlyts::get_date_col(precision=timeResolution)
names(luft_qa) <- c(names(luft_qa)[icol],paste0("qa_",names(luft_qa)[-c(icol)]))

#luft <- glow::ts(service=friendly_service,
#                 fromDate=fromDate,toDate=toDate,precision=timeResolution,timeResolution=timeResolution,
#                 element_id=element_id,station_id=station_id)

# qa indicator
#luft_qa <- glow::qa(service=friendly_service,
#                 fromDate=fromDate,toDate=toDate,precision=timeResolution,timeResolution=timeResolution,
#                 element_id=element_id,station_id=station_id)

study <- friendlyts::bind_cols_fts(luft,luft_qa,precision=precision)
study <- stats::na.omit(study)


########################
## Validation dataset  #
########################

# Period 2015-2016
fromDate <- "2015/01/01 00:00:00"
toDate <- "2016/12/31 23:00:00"

# luft
tmp <- data.luftkval.oslo10::get_short_data(stations= station_id,
                referencetimeFrom=fromDate,
                referencetimeTo=toDate,
                timeResolution=timeResolution,
                elements=element_id)

luft <- friendlyts::ts_spread(df=tmp, col_date=c("date"),
                               col_key=c("CO_NAME","ID"),col_target="value",
                               precision=timeResolution,tzone="UTC",
                               date_type="posixlt")

luft <- friendlyts::check_date(luft,
    fromDate=fromDate,
    toDate=toDate,
    timeResolution=timeResolution,
    v=1,
    precision=timeResolution,
    tzone="UTC",
    missingValues=NA,
    date_type="posixlt")

# luft qa
luft_qa <- friendlyts::ts_spread(df=tmp, col_date=c("date"),
                               col_key=c("CO_NAME","ID"),col_target="qualityCode",
                               precision=timeResolution,tzone="UTC",
                               date_type="posixlt")

luft_qa <- friendlyts::check_date(luft_qa,
    fromDate=fromDate,
    toDate=toDate,
    timeResolution=timeResolution,
    v=1,
    precision=timeResolution,
    tzone="UTC",
    missingValues=NA,
    date_type="posixlt")

icol <- friendlyts::get_date_col(precision=timeResolution)
names(luft_qa) <- c(names(luft_qa)[icol],paste0("qa_",names(luft_qa)[-c(icol)]))

#luft <- glow::ts(service=friendly_service,
#                 fromDate=fromDate,toDate=toDate,precision=timeResolution,timeResolution=timeResolution,
#                 element_id=element_id,station_id=station_id)

# qa indicator
#luft_qa <- glow::qa(service=friendly_service,
#                 fromDate=fromDate,toDate=toDate,precision=timeResolution,timeResolution=timeResolution,
#                 element_id=element_id,station_id=station_id)

validation <- friendlyts::bind_cols_fts(luft,luft_qa,precision=precision)
validation <- stats::na.omit(validation)

#########################
# Focus on valid data   #
#########################

# read qcflag nilu meta
path <- try(normalizePath(file.path(.libPaths()[1],"data.luftkval.oslo10","data","qcflag_meta.rda"),mustWork=TRUE))
tmp <- load(file=path)
qcflag_meta <- get(tmp)
known_ok   <- qcflag_meta %>% filter(FlagType=="OK") %>% select(ID)

all <- paste0(element_id,"_",station_id)
qa_all <- paste0("qa_",element_id,"_",station_id)

cond <- paste(qa_all, "%in%",  known_ok,collapse=" & ")

# valid data on train/test dataset
study <- study %>% filter(eval(parse(text=cond))) %>% select(c("WDAY","YEAR","MONTH","DAY","HOUR"),tidyselect::all_of(all),tidyselect::all_of(qa_all)) %>% friendlyts::as_tbl_friendlyts(precision=precision,date_type="friendlyts",CHECKNAS=FALSE)
study <- stats::na.omit(study)

# split 80/20
split_dataset <- cipred::partition(study,train=0.8,test=0.2)


# valid data on validation dataset
validation <- validation %>% filter(eval(parse(text=cond))) %>% select(c("WDAY","YEAR","MONTH","DAY","HOUR"),tidyselect::all_of(all),tidyselect::all_of(qa_all)) %>% friendlyts::as_tbl_friendlyts(precision=precision,date_type="friendlyts",CHECKNAS=FALSE)
validation <- stats::na.omit(validation)


###############################################
# FIT on TRAIN DATASET / PRED ON TEST DATASET #
###############################################
perf_testing <- list()
difference_testing <- list()
pred_test_eaam2010_prime <- list()
pred_test_wager2014_prime <- list()
pred_test_lu2019_prime <- list()
p_values_test <- list()

for (i_station in 1:length(station_id)) {

  # target and predictors
  station <- station_id[i_station]
  target <- paste0(element_id,"_",station)
  predictors <- setdiff(names(luft), c(c("WDAY","YEAR","MONTH","DAY","HOUR"),target,names(luft_qa)))

  cat('**********\n')
  cat(target,'\n')

  cat('-> fit ranger model\n')
  model_rf <- cipred::fit_ranger(split_dataset$train,target,predictors)

  cat('-> Ranger prediction on the test dataset\n')
  tmp <- cipred::predict_ranger(model_rf,split_dataset$test)
  pred_test <- data.frame(obs=split_dataset$test[[target]],
                          prediction=tmp$predictions)

  cat('-> metrics of Ranger on test dataset\n')
  perf_testing[[i_station]] <- sapply(c("rmse","mse","mae","crmse","ncrmse","bias","nbias","r2"),
                      FUN=function(x)
                      metrics::metrics(reference=pred_test[["obs"]],client=pred_test[["prediction"]],fun=x)  )

  perf_testing[[i_station]] <- c(ID= target,perf_testing[[i_station]])

  cat('-> difference between obs and prediction on test dataset\n')
  difference_testing[[i_station]] <- cbind(data.frame(ID= rep(target,length(pred_test[,"obs"])),obs=pred_test[["obs"]], pred=pred_test[["prediction"]], eps=metrics::differences(reference=pred_test[["obs"]],client=pred_test[["prediction"]],fun="eps")))

  cat('-> eaam2010 prediction on the test dataset\n')
  pred_test_eaam2010_prime[[i_station]] <- cipred::pred_rfci_eaamm2010_prime(model_rf,
                                                                dataset=split_dataset$test,
                                                                target=target)
  pred_test_eaam2010_prime[[i_station]] <- cbind(data.frame(ID= rep(target,length(pred_test_eaam2010_prime[[i_station]][,"obs"]))),pred_test_eaam2010_prime[[i_station]])

  cat('-> wager2014 prediction on the test dataset\n')
  pred_test_wager2014_prime[[i_station]] <- cipred::pred_rfci_wager2014_prime(model_rf,
                                                                dataset=split_dataset$test,
                                                                target=target)
  pred_test_wager2014_prime[[i_station]] <- cbind(data.frame(ID= rep(target,length(pred_test_wager2014_prime[[i_station]][,"obs"]))),pred_test_wager2014_prime[[i_station]])

  cat('-> lu2019 prediction on the test dataset\n')
  model_lu2019 <- cipred::fit_rfci_lu2019(split_dataset$train,target,predictors)
  pred_test_lu2019_prime[[i_station]] <- cipred::pred_rfci_lu2019_prime(lu2019=model_lu2019,
                                                                        dataset=split_dataset$test,
                                                                        target=target,
                                                                        predictors=predictors)
  pred_test_lu2019_prime[[i_station]] <- cbind(data.frame(ID= rep(target,length(pred_test_lu2019_prime[[i_station]][,"obs"]))),pred_test_lu2019_prime[[i_station]])

  cat('-> Gathering p-values\n')
  p_values_test[[i_station]] <- data.frame(ID = numeric(0), pvalue=numeric(0),method=character(0))
  p_values_test[[i_station]] <- rbind(p_values_test[[i_station]],data.frame(ID= rep(target,length(pred_test_eaam2010_prime[[i_station]][,"pvalue"])),pvalue=pred_test_eaam2010_prime[[i_station]][,"pvalue"],method=pred_test_eaam2010_prime[[i_station]][,"method"]))
  p_values_test[[i_station]] <- rbind(p_values_test[[i_station]],data.frame(ID= rep(target,length(pred_test_wager2014_prime[[i_station]][,"pvalue"])),pvalue=pred_test_wager2014_prime[[i_station]][,"pvalue"],method=pred_test_wager2014_prime[[i_station]][,"method"]))
  p_values_test[[i_station]] <- rbind(p_values_test[[i_station]],data.frame(ID= rep(target,length(pred_test_lu2019_prime[[i_station]][,"pvalue"])),pvalue=pred_test_lu2019_prime[[i_station]][,"pvalue"],method=pred_test_lu2019_prime[[i_station]][,"method"]))

}

perf_testing <- do.call("rbind",perf_testing)
difference_testing <- do.call("rbind",difference_testing)
p_values_test <- do.call("rbind",p_values_test)


##################################
# FIT/PRED ON VALIDATION DATASET #
##################################
perf_validation <- list()
difference_validation <- list()
pred_validation_eaam2010_prime <- list()
pred_validation_wager2014_prime <- list()
pred_validation_lu2019_prime <- list()
p_values_validation <- list()


for (i_station in 1:length(station_id)) {

  # target and predictors
  station <- station_id[i_station]
  target <- paste0(element_id,"_",station)
  predictors <- setdiff(names(luft), c(c("WDAY","YEAR","MONTH","DAY","HOUR"),target,names(luft_qa)))

  cat('**********\n')
  cat(target,'\n')

  cat('-> fit ranger model\n')
  model_rf <- cipred::fit_ranger(split_dataset$train,target,predictors)

  cat('-> Ranger prediction on the validation dataset\n')
  tmp <- cipred::predict_ranger(model_rf,validation)
  pred_validation <- data.frame(obs=validation[[target]],
                          prediction=tmp$predictions)

  cat('-> metrics of Ranger on validation dataset\n')
  perf_validation[[i_station]] <- sapply(c("rmse","mse","mae","crmse","ncrmse","bias","nbias","r2"),
                      FUN=function(x)
                      metrics::metrics(reference=pred_validation[["obs"]],client=pred_validation[["prediction"]],fun=x)  )
  perf_validation[[i_station]] <- c(ID= target,perf_validation[[i_station]])

  cat('-> difference between obs and prediction on validation dataset\n')
  difference_validation[[i_station]] <- cbind(data.frame(ID= rep(target,length(pred_validation[,"obs"])),obs=pred_validation[["obs"]], pred=pred_validation[["prediction"]],eps=metrics::differences(reference=pred_validation[["obs"]],client=pred_validation[["prediction"]],fun="eps")))


  cat('-> eaam2010 prediction on the validation dataset\n')
  pred_validation_eaam2010_prime[[i_station]] <- cipred::pred_rfci_eaamm2010_prime(model_rf,
                                                                dataset=validation,
                                                                target=target)
  pred_validation_eaam2010_prime[[i_station]] <- cbind(data.frame(ID= rep(target,length(pred_validation_eaam2010_prime[[i_station]][,"obs"]))),pred_validation_eaam2010_prime[[i_station]])

  cat('-> wager2014 prediction on the validation dataset\n')
  pred_validation_wager2014_prime[[i_station]] <- cipred::pred_rfci_wager2014_prime(model_rf,
                                                                dataset=validation,
                                                                target=target)
  pred_validation_wager2014_prime[[i_station]] <- cbind(data.frame(ID= rep(target,length(pred_validation_wager2014_prime[[i_station]][,"obs"]))),pred_validation_wager2014_prime[[i_station]])

  cat('-> lu2019 prediction on the validation dataset\n')
  model_lu2019 <- cipred::fit_rfci_lu2019(split_dataset$train,target,predictors)
  pred_validation_lu2019_prime[[i_station]] <- cipred::pred_rfci_lu2019_prime(lu2019=model_lu2019,
                                                                        dataset=validation,
                                                                        target=target,
                                                                        predictors=predictors)
  pred_validation_lu2019_prime[[i_station]] <- cbind(data.frame(ID= rep(target,length(pred_validation_lu2019_prime[[i_station]][,"obs"]))),pred_validation_lu2019_prime[[i_station]])

  cat('-> Gathering p-values\n')
  p_values_validation[[i_station]] <- data.frame(ID = numeric(0), pvalue=numeric(0),method=character(0))
  p_values_validation[[i_station]] <- rbind(p_values_validation[[i_station]],data.frame(ID= pred_validation_eaam2010_prime[[i_station]][,"ID"],pvalue=pred_validation_eaam2010_prime[[i_station]][,"pvalue"],method=pred_validation_eaam2010_prime[[i_station]][,"method"]))
  p_values_validation[[i_station]] <- rbind(p_values_validation[[i_station]],data.frame(ID= pred_validation_wager2014_prime[[i_station]][,"ID"],pvalue=pred_validation_wager2014_prime[[i_station]][,"pvalue"],method=pred_validation_wager2014_prime[[i_station]][,"method"]))
  p_values_validation[[i_station]] <- rbind(p_values_validation[[i_station]],data.frame(ID= pred_validation_lu2019_prime[[i_station]][,"ID"],pvalue=pred_validation_lu2019_prime[[i_station]][,"pvalue"],method=pred_validation_lu2019_prime[[i_station]][,"method"]))

}
perf_validation <- do.call("rbind",perf_validation)
difference_validation <- do.call("rbind",difference_validation)
p_values_validation <- do.call("rbind",p_values_validation)

####################################################################################
#################### CHARTS ##########################################################
####################################################################################

###############################################################
# figure pqqplot including the three methods                  #
###############################################################
cipred::pqq_plot(df=p_values_test,DESKTOP=FALSE,name="oslo_testing", path='~')
cipred::pqq_plot(df=p_values_validation,DESKTOP=FALSE,name="oslo_validation", path='~')


#################################
# flattening the lists          #
#################################
pred_validation_eaam2010_prime <- do.call("rbind",pred_validation_eaam2010_prime)
pred_validation_wager2014_prime <- do.call("rbind",pred_validation_wager2014_prime)
pred_validation_lu2019_prime <- do.call("rbind",pred_validation_lu2019_prime)

#######################################################################
# Illustrative figure of observation, prediction and CI95% Eaamm2010  #
#######################################################################
res <- cbind(validation,pred_validation_eaam2010_prime %>% filter(ID=="NO2_7"))
res <- plotfts::sub_period(tbl_fts=res,fromDate="2016/12/04 00:00:00",toDate="2016/12/08 00:00:00",timeResolution=timeResolution,tzone="UTC")
res <- friendlyts::dfts(tbl_fts=res,timeResolution=timeResolution,tzone="UTC")

res <- dplyr::select(res,c("date","NO2_7",prediction,lower_pred_0_05,upper_pred_0_05))
names(res) <- c("date","NO2_7","prediction","lower_pred_0_05","upper_pred_0_05")

filename_x <- normalizePath(file.path("~","timeseries_eaamm2010.tiff"),mustWork = FALSE)

grDevices::tiff(filename = filename_x, width=200,height=200,
      units = "mm", pointsize = 12,
      compression = "lzw",
      bg = "transparent", res = 300)

p <- ggplot2::ggplot(res, ggplot2::aes(date)) +
     ggplot2::geom_ribbon(ggplot2::aes(ymin = lower_pred_0_05, ymax = upper_pred_0_05), fill = "grey80") +
     ggplot2::geom_point(ggplot2::aes(y = NO2_7)) +
     ggplot2::geom_line(ggplot2::aes(y = prediction)) +
     ggplot2::labs(x = "date",y = "[ug/m^3]")

p

grDevices::dev.off()

#######################################################################
# Illustrative figure of observation, prediction and CI95% Wager2014  #
#######################################################################
res <- cbind(validation,pred_validation_wager2014_prime %>% filter(ID=="NO2_7"))
res <- plotfts::sub_period(tbl_fts=res,fromDate="2016/12/04 00:00:00",toDate="2016/12/08 00:00:00",timeResolution=timeResolution,tzone="UTC")
res <- friendlyts::dfts(tbl_fts=res,timeResolution=timeResolution,tzone="UTC")

res <- dplyr::select(res,c("date","NO2_7",prediction,lower_pred_0_05,upper_pred_0_05))
names(res) <- c("date","NO2_7","prediction","lower_pred_0_05","upper_pred_0_05")

filename_x <- normalizePath(file.path("~","timeseries_wager2014.tiff"),mustWork = FALSE)

grDevices::tiff(filename = filename_x, width=200,height=200,
      units = "mm", pointsize = 12,
      compression = "lzw",
      bg = "transparent", res = 300)

p <- ggplot2::ggplot(res, ggplot2::aes(date)) +
     ggplot2::geom_ribbon(ggplot2::aes(ymin = lower_pred_0_05, ymax = upper_pred_0_05), fill = "grey80") +
     ggplot2::geom_point(ggplot2::aes(y = NO2_7)) +
     ggplot2::geom_line(ggplot2::aes(y = prediction)) +
     ggplot2::labs(x = "date",y = "[ug/m^3]")

p

grDevices::dev.off()

#######################################################################
# Illustrative figure of observation, prediction and CI95% Lu2019  #
#######################################################################
res <- cbind(validation,pred_validation_lu2019_prime %>% filter(ID=="NO2_7"))
res <- plotfts::sub_period(tbl_fts=res,fromDate="2016/12/04 00:00:00",toDate="2016/12/08 00:00:00",timeResolution=timeResolution,tzone="UTC")
res <- friendlyts::dfts(tbl_fts=res,timeResolution=timeResolution,tzone="UTC")

res <- dplyr::select(res,c("date","NO2_7",prediction,lower_pred_0_05,upper_pred_0_05))
names(res) <- c("date","NO2_7","prediction","lower_pred_0_05","upper_pred_0_05")

filename_x <- normalizePath(file.path("~","timeseries_lu2019.tiff"),mustWork = FALSE)

grDevices::tiff(filename = filename_x, width=200,height=200,
      units = "mm", pointsize = 12,
      compression = "lzw",
      bg = "transparent", res = 300)

p <- ggplot2::ggplot(res, ggplot2::aes(date)) +
     ggplot2::geom_ribbon(ggplot2::aes(ymin = lower_pred_0_05, ymax = upper_pred_0_05), fill = "grey80") +
     ggplot2::geom_point(ggplot2::aes(y = NO2_7)) +
     ggplot2::geom_line(ggplot2::aes(y = prediction)) +
     ggplot2::labs(x = "date",y = "[ug/m^3]")

p

grDevices::dev.off()


#######################################################################
# Illustrative figure of observation, prediction and CI95% Lu2019  #
#######################################################################
res <- cbind(validation,pred_validation_lu2019_prime %>% filter(ID=="NO2_11"))
res <- plotfts::sub_period(tbl_fts=res,fromDate="2016/12/04 00:00:00",toDate="2016/12/08 00:00:00",timeResolution=timeResolution,tzone="UTC")
res <- friendlyts::dfts(tbl_fts=res,timeResolution=timeResolution,tzone="UTC")

res <- dplyr::select(res,c("date","NO2_11",prediction,lower_pred_0_05,upper_pred_0_05))
names(res) <- c("date","NO2_11","prediction","lower_pred_0_05","upper_pred_0_05")

filename_x <- normalizePath(file.path("~","timeseries_lu2019_NO2_11.tiff"),mustWork = FALSE)

grDevices::tiff(filename = filename_x, width=200,height=200,
      units = "mm", pointsize = 12,
      compression = "lzw",
      bg = "transparent", res = 300)

p <- ggplot2::ggplot(res, ggplot2::aes(date)) +
     ggplot2::geom_ribbon(ggplot2::aes(ymin = lower_pred_0_05, ymax = upper_pred_0_05), fill = "grey80") +
     ggplot2::geom_point(ggplot2::aes(y = NO2_11)) +
     ggplot2::geom_line(ggplot2::aes(y = prediction)) +
     ggplot2::labs(x = "date",y = "[ug/m^3]")

p

grDevices::dev.off()
